extension StringExtensions on String? {
  bool isNullOrEmpty() => this?.isEmpty ?? true;

  String emptyIfNull() => this ?? "";

  String defaultIfEmpty(String defaultVal) => isNullOrEmpty() ? defaultVal.emptyIfNull() : this!;

  String? capitalize() => isNullOrEmpty() ? this : '${this![0].toUpperCase()}${this!.substring(1)}';

  bool equalsIgnoreCase(String? lookup) {
    return this?.toLowerCase() == lookup?.toLowerCase();
  }

  bool containsIgnoreCase(String? lookup) {
    return lookup != null && this!.toLowerCase().contains(lookup.toLowerCase());
  }

  String truncateWithEllipsis(int maxLength) {
    if (this == null || this!.length <= maxLength) return this ?? '';
    return '${this!.substring(0, maxLength - 3)}...';
  }

  /// Returns: <br>
  /// [empty] if the value is null,
  /// [current value] if only a single word
  /// [formal name]
  /// <br><br>
  /// eg: [null.toFormalName() => ' ']<br>
  /// eg: ['stubbly'.toFormalName() => 'stubbly']<br>
  /// eg: ['Matthew Dodds'.toFormalName() => 'M Dodds']
  String toFormalName() {
    if (isNullOrEmpty()) return '';
    final split = this!.split(' ');
    if (split.length == 1) return this!;
    return '${split.sublist(0, split.length -1).map((n) => n[0]).join(' ')} ${split[split.length -1]}';
  }

}